import logging
from unittest import TestCase, skipIf

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestQuota(HvacIntegrationTestCase, TestCase):
    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(backend_type="kv", path="test")

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path="test")
        super().tearDown()

    @skipIf(
        utils.vault_version_ge("1.12.0"),
        "Older versions of Vault return different JSON structure",
    )
    def test_create_quota_old(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*/",
            "rate": 100,
            "type": "rate-limit",
        }

        # Create quota
        create_or_update_quota_response = self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )
        logging.debug(
            "create_or_update_quota_response: %s" % create_or_update_quota_response
        )

        # API takes str arg for interval and block_interval but returns attribute in int num of seconds
        quota_dict["block_interval"] = int(quota_dict["block_interval"])
        quota_dict["interval"] = int(quota_dict["interval"])

        self.assertEqual(
            first=quota_dict,
            second=self.client.sys.read_quota(quota_dict["name"])["data"],
        )

        with self.assertRaises(exceptions.InvalidRequest):
            self.client.sys.create_or_update_quota(
                name="test-invalid-path", rate=101, path="/not-exist"
            )

    @skipIf(
        utils.vault_version_lt("1.12.0") or utils.vault_version_ge("1.15.0"),
        "Newer version of quota JSON changes path structure and adds role. Route only works on enterprise from 1.15 onwards",
    )
    def test_create_quota(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*",
            "rate": 100,
            "role": "",
            "type": "rate-limit",
        }

        # Create quota
        create_or_update_quota_response = self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )
        logging.debug(
            "create_or_update_quota_response: %s" % create_or_update_quota_response
        )

        # API takes str arg for interval and block_interval but returns attribute in int num of seconds
        quota_dict["block_interval"] = int(quota_dict["block_interval"])
        quota_dict["interval"] = int(quota_dict["interval"])

        self.assertEqual(
            first=quota_dict,
            second=self.client.sys.read_quota(quota_dict["name"])["data"],
        )

        with self.assertRaises(exceptions.InvalidRequest):
            self.client.sys.create_or_update_quota(
                name="test-invalid-path", rate=101, path="/not-exist"
            )

    @skipIf(
        utils.vault_version_ge("1.12.0"),
        "Older versions of Vault return different JSON structure",
    )
    def test_update_quota_old(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*/",
            "rate": 100,
            "type": "rate-limit",
        }

        # Create quota
        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=101,
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # Update quota
        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # API takes str arg for interval and block_interval but returns attribute in int num of seconds
        quota_dict["block_interval"] = int(quota_dict["block_interval"])
        quota_dict["interval"] = int(quota_dict["interval"])

        self.assertEqual(
            first=quota_dict,
            second=self.client.sys.read_quota(quota_dict["name"])["data"],
        )

    @skipIf(
        utils.vault_version_lt("1.12.0") or utils.vault_version_ge("1.15.0"),
        "Newer version of quota JSON changes path structure and adds role. Route only works on enterprise from 1.15 onwards",
    )
    def test_update_quota(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*",
            "rate": 100,
            "role": "",
            "type": "rate-limit",
        }

        # Create quota
        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=101,
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # Update quota
        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # API takes str arg for interval and block_interval but returns attribute in int num of seconds
        quota_dict["block_interval"] = int(quota_dict["block_interval"])
        quota_dict["interval"] = int(quota_dict["interval"])

        self.assertEqual(
            first=quota_dict,
            second=self.client.sys.read_quota(quota_dict["name"])["data"],
        )

    @skipIf(
        utils.vault_version_ge("1.12.0"),
        "Older versions of Vault return different JSON structure",
    )
    def test_read_quota_old(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*/",
            "rate": 100,
            "type": "rate-limit",
        }

        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # Read the quota that was just created
        read_quota_response = self.client.sys.read_quota(
            name=quota_dict["name"],
        )
        logging.debug("read_quota_response: %s" % read_quota_response)

        # API takes str arg for interval and block_interval but returns attribute in int num of seconds
        quota_dict["block_interval"] = int(quota_dict["block_interval"])
        quota_dict["interval"] = int(quota_dict["interval"])

        self.assertEqual(
            first=quota_dict,
            second=read_quota_response["data"],
        )

    @skipIf(
        utils.vault_version_lt("1.12.0") or utils.vault_version_ge("1.15.0"),
        "Newer version of quota JSON changes path structure and adds role. Route only works on enterprise from 1.15 onwards",
    )
    def test_read_quota(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*",
            "rate": 100,
            "role": "",
            "type": "rate-limit",
        }

        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # Read the quota that was just created
        read_quota_response = self.client.sys.read_quota(
            name=quota_dict["name"],
        )
        logging.debug("read_quota_response: %s" % read_quota_response)

        # API takes str arg for interval and block_interval but returns attribute in int num of seconds
        quota_dict["block_interval"] = int(quota_dict["block_interval"])
        quota_dict["interval"] = int(quota_dict["interval"])

        self.assertEqual(
            first=quota_dict,
            second=read_quota_response["data"],
        )

    @skipIf(
        utils.vault_version_lt("1.12.0") or utils.vault_version_ge("1.15.0"),
        "Newer version of quota JSON changes path structure and adds role. Route only works on enterprise from 1.15 onwards",
    )
    def test_list_quotas(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*",
            "rate": 100,
            "role": "",
            "type": "rate-limit",
        }

        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        list_quotas_response = self.client.sys.list_quotas()
        logging.debug("list_quotas_response: %s" % list_quotas_response)

        # API takes str arg for interval and block_interval but returns attribute in int num of seconds
        quota_dict["block_interval"] = int(quota_dict["block_interval"])
        quota_dict["interval"] = int(quota_dict["interval"])

        self.assertIn(
            member="test-quota",
            container=list_quotas_response["data"]["keys"],
        )

    @skipIf(
        utils.vault_version_ge("1.12.0"),
        "Older versions of Vault return different JSON structure",
    )
    def test_delete_quota_old(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*/",
            "rate": 100,
            "type": "rate-limit",
        }

        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # Delete the quota that was just created
        delete_quota_response = self.client.sys.delete_quota(
            name=quota_dict["name"],
        )

        logging.debug("delete_quota_response: %s" % delete_quota_response)

        with self.assertRaises(exceptions.InvalidPath):
            self.client.sys.read_quota(
                name=quota_dict["name"],
            )

    @skipIf(
        utils.vault_version_lt("1.12.0") or utils.vault_version_ge("1.15.0"),
        "Newer version of quota JSON changes path structure and adds role. Route only works on enterprise from 1.15 onwards",
    )
    def test_delete_quota(self):
        quota_dict = {
            "block_interval": "600",
            "interval": "600",
            "name": "test-quota",
            "path": "test/*",
            "rate": 100,
            "role": "",
            "type": "rate-limit",
        }

        self.client.sys.create_or_update_quota(
            name=quota_dict["name"],
            rate=quota_dict["rate"],
            path=quota_dict["path"],
            interval=quota_dict["interval"],
            block_interval=quota_dict["block_interval"],
        )

        # Delete the quota that was just created
        delete_quota_response = self.client.sys.delete_quota(
            name=quota_dict["name"],
        )

        logging.debug("delete_quota_response: %s" % delete_quota_response)

        with self.assertRaises(exceptions.InvalidPath):
            self.client.sys.read_quota(
                name=quota_dict["name"],
            )



#!/usr/bin/env python
import logging
import os
import subprocess
import time
import requests
import hcl
import typing as t

import shutil
from unittest import SkipTest
from tests.utils import (
    get_config_file_path,
    load_config_file,
    create_client,
    PortGetter,
)

from hvac.v1 import Client

from requests.adapters import HTTPAdapter
from urllib3.util import Retry

logger = logging.getLogger(__name__)


class TestProcessInfo:
    name: str
    process: subprocess.Popen
    extra: t.List[str]

    def __init__(
        self, name: str, process: subprocess.Popen, *extra: t.List[str]
    ) -> None:
        self.name = name
        self.process = process
        self.extra = extra

    def log_name(self, index: int, *suffixes: t.List[str], ext: str = ".log"):
        segmented = "_".join([self.name, str(index), *self.extra, *suffixes])
        return f"{segmented}{ext}"


class ServerManager:
    """Runs vault process running with test configuration and associates a hvac Client instance with this process."""

    active_config_paths: t.Optional[t.List[str]]
    config_paths: t.List[str]
    client: t.Optional[Client]
    use_consul: bool
    patch_config: bool

    def __init__(
        self,
        config_paths: t.List[str],
        client: Client = None,
        use_consul: bool = False,
        patch_config: bool = True,
    ):
        """Set up class attributes for managing a vault server process.

        :param config_paths: Full path to the Vault config to use when launching `vault server`.
        :type config_paths: list[str]
        :param client: Hvac Client that is used to initialize the vault server process.
        :type client: hvac.v1.Client
        """

        self.active_config_paths = None
        self.config_paths = config_paths
        self.client = client
        self.use_consul = use_consul
        self.patch_config = patch_config

        self.keys = None
        self.root_token = None

        self._processes: t.List[TestProcessInfo] = []

    def patch_config_port(
        self,
        config_file: str,
        *,
        port_getter: PortGetter.PortGetterProtocol,
        insert: bool = False,
        address: str = None,
        additional_sections: t.Optional[t.Dict[str, t.Any]] = None,
        output_dir: str = "generated",
    ):
        worker = os.getenv("PYTEST_XDIST_WORKER", "solo")
        config_parent = os.path.dirname(config_file)
        if not os.path.isabs(output_dir):
            output_dir = os.path.join(config_parent, output_dir)
        output_file = os.path.join(
            output_dir, os.path.basename(config_file).replace(".hcl", f"_{worker}.json")
        )

        with open(config_file, "r") as f:
            config: dict = hcl.load(f)

        if "listener" in config:
            listeners = config["listener"]
            if not isinstance(listeners, list):
                listeners = [listeners]
            for linstances in listeners:
                if "tcp" in linstances:
                    listener = linstances["tcp"]
                    if "address" in listener:
                        addr, port = listener["address"].split(":")
                        if address is not None:
                            addr = address
                        addr, port = port_getter(address=addr, port=int(port))
                        listener["address"] = ":".join((addr, str(port)))
                    elif insert:
                        addr, port = port_getter(address=address)
                        listener["address"] = ":".join((addr, str(port)))

        if additional_sections is not None:
            config.update(additional_sections)

        with open(output_file, "w") as f:
            hcl.api.json.dump(config, f, indent=4)

        return output_file

    def start(self):
        consul_config = None
        if self.use_consul:
            consul_addr = self.start_consul()
            consul_config = {
                "storage": {
                    "consul": {
                        "address": consul_addr,
                        "path": "vault_whatever/",
                    }
                }
            }
        self.start_vault(consul_config=consul_config)

    def start_vault(
        self, *, consul_config: dict = None, attempt=1, max_attempts=3, delay_s=1
    ):
        """Launch the vault server process and wait until its online and ready."""
        if shutil.which("vault") is None:
            raise SkipTest("Vault executable not found")

        with PortGetter() as g:
            self.active_config_paths = [
                self.patch_config_port(
                    config_path,
                    port_getter=g.get_port,
                    insert=True,
                    additional_sections=consul_config,
                )
                if self.patch_config
                else config_path
                for config_path in self.config_paths
            ]

        cluster_ready = False
        for config_path in self.active_config_paths:
            this_addr = self.get_config_vault_address(config_path)
            this_client = create_client(url=this_addr)
            if self.client is None:
                self.client = this_client

            command = ["vault", "server", "-config=" + config_path]
            logger.debug(f"Starting vault server with command: {command}")
            process = subprocess.Popen(
                command, stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
            self._processes.append(
                TestProcessInfo("vault", process, os.path.basename(config_path))
            )
            logger.debug(f"Spawned vault server with PID {process.pid}")

            attempts_left = 20
            last_exception = None
            while attempts_left > 0 and not cluster_ready:
                try:
                    logger.debug("Checking if vault is ready...")
                    this_client.sys.is_initialized()
                    cluster_ready = True
                    break
                except Exception as ex:
                    if process.poll() is not None:
                        stdout, stderr = process.stdout, process.stderr
                        if attempt < max_attempts:
                            logger.debug(
                                f"Starting Vault failed (attempt {attempt} of {max_attempts}):\n{last_exception}\n{stdout.readlines()}\n{stderr.readlines()}"
                            )
                            time.sleep(delay_s)
                            self.start_vault(
                                attempt=(attempt + 1),
                                max_attempts=max_attempts,
                                delay_s=delay_s,
                            )
                        else:
                            raise Exception(
                                "Vault server terminated before becoming ready"
                            )
                    logger.debug("Waiting for Vault to start")
                    time.sleep(0.5)
                    attempts_left -= 1
                    last_exception = ex
            if not cluster_ready:
                if process.poll() is None:
                    process.kill()
                stdout, stderr = process.communicate()
                if attempt < max_attempts:
                    logger.debug(
                        f"Vault never became ready (attempt {attempt} of {max_attempts}):\n{last_exception}\n{stdout}\n{stderr}"
                    )
                    time.sleep(delay_s)
                    self.start_vault(
                        attempt=(attempt + 1),
                        max_attempts=max_attempts,
                        delay_s=delay_s,
                    )
                else:
                    raise Exception(
                        "Unable to start Vault in background:\n{err}\n{stdout}\n{stderr}".format(
                            err=last_exception,
                            stdout=stdout,
                            stderr=stderr,
                        )
                    )

    def start_consul(
        self,
    ) -> str:
        if shutil.which("consul") is None:
            raise SkipTest("Consul executable not found")

        with PortGetter() as g:
            http_addr, http_port = g.get_port()
            _, server_port = g.get_port(address=http_addr)
            _, serf_lan_port = g.get_port(address=http_addr)
            _, serf_wan_port = g.get_port(address=http_addr)
            consul_addr = f"{http_addr}:{http_port}"
            command = [
                "consul",
                "agent",
                "-dev",
                "-disable-host-node-id",
                f"-serf-lan-port={serf_lan_port}",
                f"-serf-wan-port={serf_wan_port}",
                f"-server-port={server_port}",
                "-grpc-port=-1",
                "-grpc-tls-port=-1",
                f"-bind={http_addr}",
                f"-http-port={http_port}",
                "-dns-port=-1",
            ]

        logger.debug(f"Starting consul service with command: {command}")
        process = subprocess.Popen(
            command, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        self._processes.append(
            TestProcessInfo("consul", process, os.getenv("PYTEST_XDIST_WORKER", "solo"))
        )
        attempts_left = 20
        last_exception = None
        while attempts_left > 0:
            try:
                catalog_nodes_response = requests.get(
                    f"http://{consul_addr}/v1/catalog/nodes"
                )
                nodes_list = catalog_nodes_response.json()
                logger.debug(
                    "JSON response from request to consul/v1/catalog/nodes: {resp}".format(
                        resp=nodes_list
                    )
                )
                node_name = nodes_list[0]["Node"]
                logger.debug(f"Current consul node name: {node_name}")
                node_health_response = requests.get(
                    f"http://{consul_addr}/v1/health/node/{node_name}"
                )
                node_health = node_health_response.json()
                logger.debug(f"Node health response: {node_health}")
                assert (
                    node_health[0]["Status"] == "passing"
                ), f'Node {node_name} status != "passing"'
                return consul_addr
            except Exception as error:
                if process.poll() is not None:
                    raise Exception("Consul service terminated before becoming ready")
                logger.debug(
                    "Unable to connect to consul while waiting for process to start: {err}".format(
                        err=error
                    )
                )
                time.sleep(0.5)
                attempts_left -= 1
                last_exception = error

        raise Exception(f"Unable to start consul in background: {last_exception}")

    def stop(self):
        """Stop the vault server process being managed by this class."""
        self.client = None
        for process_num, pinfo in reversed(list(enumerate(self._processes))):
            logger.debug(
                f"Terminating {pinfo.name} server with PID {pinfo.process.pid}"
            )
            if pinfo.process.poll() is None:
                pinfo.process.kill()

            if os.getenv("HVAC_OUTPUT_VAULT_STDERR", False):
                try:
                    stdout_lines, stderr_lines = pinfo.process.communicate()
                except ValueError:
                    pass
                else:
                    log_dir = get_config_file_path("generated", "logs")
                    try:
                        os.mkdir(log_dir)
                    except FileExistsError:
                        pass
                    stderr_filename = pinfo.log_name(process_num, "stderr")
                    stderr_path = get_config_file_path(log_dir, stderr_filename)
                    with open(stderr_path, "w") as f:
                        logger.debug(stderr_lines.decode())
                        f.writelines(stderr_lines.decode())
                    stdout_filename = pinfo.log_name(process_num, "stdout")
                    stdout_path = get_config_file_path(log_dir, stdout_filename)
                    with open(get_config_file_path(stdout_path), "w") as f:
                        logger.debug(stdout_lines.decode())
                        f.writelines(stdout_lines.decode())

    def initialize(self):
        """Perform initialization of the vault server process and record the provided unseal keys and root token."""
        if self.client.sys.is_initialized():
            raise RuntimeError(
                f"Vault is already initialized: {self.get_active_vault_addresses()}"
            )

        result = self.client.sys.initialize(secret_shares=5, secret_threshold=3)

        self.root_token = self.client.token = result["root_token"]
        self.keys = result["keys"]

    def restart_vault_cluster(self, perform_init=True):
        self.stop()
        self.start()
        if perform_init:
            self.initialize()

    def get_config_vault_address(self, config_path: str) -> str:
        config_hcl = load_config_file(config_path)
        config = hcl.loads(config_hcl)
        try:
            vault_address = "https://{addr}".format(
                addr=config["listener"]["tcp"]["address"]
            )
        except KeyError as error:
            logger.debug(
                "Unable to find explicit Vault address in config file {path}: {err}".format(
                    path=config_path,
                    err=error,
                )
            )
            vault_address = "https://127.0.0.1:8200"
            logger.debug(f"Using default address: {vault_address}")
        return vault_address

    def get_active_vault_addresses(self):
        vault_addresses = []
        config_paths = (
            self.active_config_paths
            if self.active_config_paths is not None
            else self.config_paths
        )
        for config_path in config_paths:
            vault_addresses.append(self.get_config_vault_address(config_path))
        return vault_addresses

    def unseal(self):
        """Unseal the vault server process."""
        vault_addresses = self.get_active_vault_addresses()
        for vault_address in vault_addresses:
            # At this point, the vault server may not be ready yet, resulting in "Connection refused"
            # failures for requests. Let's retry multiple times before giving up.
            adapter = HTTPAdapter(
                max_retries=Retry(total=3, connect=3, backoff_factor=0.1)
            )
            session = requests.Session()
            session.mount("http://", adapter)
            session.mount("https://", adapter)

            client = create_client(url=vault_address, session=session)
            client.sys.submit_unseal_keys(self.keys)



import pytest

from hvac.exceptions import ParamValidationError
from hvac.api.system_backend.init import Init


@pytest.fixture
def sys_init(mock_adapter):
    return Init(mock_adapter)


INIT_SECRET_PGP_ERROR_MSG = (
    r"length of pgp_keys list argument must equal secret_shares value"
)
INIT_RECOVERY_PGP_ERROR_MSG = (
    r"length of recovery_pgp_keys list argument must equal recovery_shares value"
)
INIT_RECOVERY_SHARES_ERROR_MSG = r"value for recovery_threshold argument must be less than or equal to recovery_shares argument"
INIT_STORED_SHARES_ERROR_MSG = (
    r"value for stored_shares argument must equal secret_shares argument"
)


class TestInit:
    @pytest.mark.parametrize(
        ["secret_shares", "recovery_shares", "expected_value", "expected_warn"],
        [
            (None, None, 5, True),
            (3, None, 3, False),
            (5, 7, 5, False),
            (None, 9, None, False),
        ],
    )
    def test_initialize_default_secret_shares(
        self,
        sys_init,
        mock_warn,
        secret_shares,
        recovery_shares,
        expected_value,
        expected_warn,
    ):
        (r_args, r_kwargs) = sys_init.initialize(
            secret_shares=secret_shares,
            recovery_shares=recovery_shares,
            recovery_threshold=0,
        )
        params = r_kwargs["json"]
        assert params["secret_shares"] == expected_value

        if expected_warn:
            mock_warn.assert_called_once()
        else:
            mock_warn.assert_not_called()

    @pytest.mark.parametrize(
        ["secret_threshold", "recovery_threshold", "expected_value", "expected_warn"],
        [
            (None, None, 3, True),
            (3, None, 3, False),
            (5, 7, 5, False),
            (None, 9, None, False),
        ],
    )
    def test_initialize_default_secret_threshold(
        self,
        sys_init,
        mock_warn,
        secret_threshold,
        recovery_threshold,
        expected_value,
        expected_warn,
    ):
        (r_args, r_kwargs) = sys_init.initialize(
            secret_threshold=secret_threshold,
            recovery_threshold=recovery_threshold,
            secret_shares=0,
        )
        params = r_kwargs["json"]
        assert params["secret_threshold"] == expected_value

        if expected_warn:
            mock_warn.assert_called_once()
        else:
            mock_warn.assert_not_called()

    @pytest.mark.parametrize(
        [
            "secret_shares",
            "pgp_keys",
            "stored_shares",
            "recovery_shares",
            "recovery_pgp_keys",
            "recovery_threshold",
            "exc_msg",
        ],
        [
            (
                2,
                [1, 2, 3],
                2,
                None,
                None,
                None,
                INIT_SECRET_PGP_ERROR_MSG,
            ),
            (
                2,
                [1, 2, 3],
                3,
                None,
                None,
                None,
                INIT_SECRET_PGP_ERROR_MSG,
            ),
            (
                2,
                [1, 2],
                3,
                None,
                None,
                None,
                INIT_STORED_SHARES_ERROR_MSG,
            ),
            (2, [1, 2], 2, 3, [1, 2], None, INIT_RECOVERY_PGP_ERROR_MSG),
            (2, [1, 2], 2, 3, [1, 2], 1, INIT_RECOVERY_PGP_ERROR_MSG),
            (2, [1, 2], 2, 3, [1, 2], 9, INIT_RECOVERY_SHARES_ERROR_MSG),
        ],
    )
    def test_initialize_errors(
        self,
        sys_init,
        mock_adapter,
        secret_shares,
        pgp_keys,
        stored_shares,
        recovery_shares,
        recovery_pgp_keys,
        recovery_threshold,
        exc_msg,
    ):
        with pytest.raises(ParamValidationError, match=exc_msg):
            sys_init.initialize(
                secret_threshold=0,  # TODO(v3.0.0): remove this, only set to suppress warning
                secret_shares=secret_shares,
                pgp_keys=pgp_keys,
                stored_shares=stored_shares,
                recovery_shares=recovery_shares,
                recovery_pgp_keys=recovery_pgp_keys,
                recovery_threshold=recovery_threshold,
            )

        mock_adapter.request.assert_not_called()

    def test_initialize_value_pass(self, sys_init):
        (r_args, r_kwargs) = sys_init.initialize(
            secret_threshold=0,
            secret_shares=2,
            root_token_pgp_key="abc",
            pgp_keys=[1, 2],
            stored_shares=2,
            recovery_shares=3,
            recovery_pgp_keys=[1, 2, 3],
            recovery_threshold=3,
        )
        params = r_kwargs["json"]

        assert params["secret_threshold"] == 0
        assert params["secret_shares"] == 2
        assert params["root_token_pgp_key"] == "abc"
        assert params["pgp_keys"] == [1, 2]
        assert params["stored_shares"] == 2
        assert params["recovery_shares"] == 3
        assert params["recovery_pgp_keys"] == [1, 2, 3]
        assert params["recovery_threshold"] == 3



from unittest import TestCase, skipIf

from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestRaft(HvacIntegrationTestCase, TestCase):
    @skipIf(
        not utils.is_enterprise() or utils.vault_version_lt("1.8.0"),
        "Raft automated snapshots only supported in Enterprise Vault",
    )
    def test_create_raft_auto_config(self):
        raft_configs_dict = {
            "name": "my-local-auto-snapshot",
            "interval": "86400",
            "storage_type": "local",
            "retain": 5,
            "local_max_space": "100000",
            "path_prefix": "/opt/vault/backups",
            "file_prefix": "vault-raft-auto-snapshot",
        }

        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name=raft_configs_dict["name"],
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        self.assertEqual(
            first=raft_configs_dict,
            second=self.client.sys.read_raft_auto_snapshot_config(
                raft_configs_dict["name"]
            )["data"],
        )

    @skipIf(
        not utils.is_enterprise() or utils.vault_version_lt("1.8.0"),
        "Raft automated snapshots only supported in Enterprise Vault",
    )
    def test_update_raft_auto_config(self):
        raft_configs_dict = {
            "name": "my-local-auto-snapshot",
            "interval": "86400",
            "storage_type": "local",
            "retain": 5,
            "local_max_space": "100000",
            "path_prefix": "/opt/vault/backups",
            "file_prefix": "vault-raft-auto-snapshot",
        }

        # Create initial configuration
        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name=raft_configs_dict["name"],
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        raft_configs_dict["path_prefix"] = "/opt/vault/backups2"

        # Update configuration
        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name=raft_configs_dict["name"],
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        self.assertEqual(
            first=raft_configs_dict,
            second=self.client.sys.read_raft_auto_snapshot_config(
                raft_configs_dict["name"]
            )["data"],
        )

    @skipIf(
        not utils.is_enterprise() or utils.vault_version_lt("1.8.0"),
        "Raft automated snapshots only supported in Enterprise Vault",
    )
    def test_read_raft_auto_config(self):
        raft_configs_dict = {
            "name": "my-local-auto-snapshot",
            "interval": "86400",
            "storage_type": "local",
            "retain": 5,
            "local_max_space": "100000",
            "path_prefix": "/opt/vault/backups",
            "file_prefix": "vault-raft-auto-snapshot",
        }

        # Create initial configuration
        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name=raft_configs_dict["name"],
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        self.assertEqual(
            first=raft_configs_dict,
            second=self.client.sys.read_raft_auto_snapshot_config(
                raft_configs_dict["name"]
            )["data"],
        )

    @skipIf(
        not utils.is_enterprise() or utils.vault_version_lt("1.8.0"),
        "Raft automated snapshots only supported in Enterprise Vault",
    )
    def test_list_raft_auto_configs(self):
        raft_configs_dict = {
            "name": "my-local-auto-snapshot",
            "interval": "86400",
            "storage_type": "local",
            "retain": 5,
            "local_max_space": "100000",
            "path_prefix": "/opt/vault/backups",
            "file_prefix": "vault-raft-auto-snapshot",
        }

        # Create initial configuration
        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name=raft_configs_dict["name"],
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        self.assertIn(
            member=raft_configs_dict["name"],
            container=self.client.sys.list_raft_auto_snapshot_configs()["data"]["keys"],
        )

    @skipIf(
        not utils.is_enterprise() or utils.vault_version_lt("1.8.0"),
        "Raft automated snapshots only supported in Enterprise Vault",
    )
    def test_delete_raft_auto_config(self):
        raft_configs_dict = {
            "name": "my-local-auto-snapshot",
            "interval": "86400",
            "storage_type": "local",
            "retain": 5,
            "local_max_space": "100000",
            "path_prefix": "/opt/vault/backups",
            "file_prefix": "vault-raft-auto-snapshot",
        }

        # Create initial configuration
        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name=raft_configs_dict["name"],
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        # Path and file prefix can't be shared between configs
        raft_configs_dict["file_prefix"] = "vault-raft-auto-snapshot2"
        raft_configs_dict["path_prefix"] = "/opt/vault/backups2"

        # Create a second config because list endpoint raises a 404 if no configs exist
        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name="my-local-auto-snapshot2",
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        self.client.sys.delete_raft_auto_snapshot_config(name=raft_configs_dict["name"])

        self.assertNotIn(
            member=raft_configs_dict,
            container=self.client.sys.list_raft_auto_snapshot_configs()["data"]["keys"],
        )

    @skipIf(
        not utils.is_enterprise() or utils.vault_version_lt("1.8.0"),
        "Raft automated snapshots only supported in Enterprise Vault",
    )
    def test_read_auto_snapshot_status(self):
        raft_configs_dict = {
            "name": "my-local-auto-snapshot",
            "interval": "86400",
            "storage_type": "local",
            "retain": 5,
            "local_max_space": "100000",
            "path_prefix": "/opt/vault/backups",
            "file_prefix": "vault-raft-auto-snapshot",
        }

        # Create initial configuration
        self.client.sys.create_or_update_raft_auto_snapshot_config(
            name=raft_configs_dict["name"],
            interval=raft_configs_dict["interval"],
            storage_type=raft_configs_dict["storage_type"],
            retain=raft_configs_dict["retain"],
            local_max_space=raft_configs_dict["local_max_space"],
            path_prefix=raft_configs_dict["path_prefix"],
            file_prefix=raft_configs_dict["file_prefix"],
        )

        # Confirm that key "data" exists within the response
        self.assertIn(
            member="data",
            container=self.client.sys.read_raft_auto_snapshot_status(
                raft_configs_dict["name"]
            ),
        )



from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac.adapters import JSONAdapter
from hvac.api.secrets_engines import Ldap
from hvac.api.secrets_engines.ldap import DEFAULT_MOUNT_POINT


class TestLdap(TestCase):
    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_configure(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/{mount_point}/config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.configure(
            binddn="cn=admin,dc=example,dc=com",
            bindpass="password",
            url="ldaps://ldap.example.com",
            mount_point=mount_point,
            upndomain="example.com",
            password_policy=None,
            userattr=None,
            schema=None,
            userdn="ou=users,dc=example,dc=com",
            connection_timeout="60s",
            request_timeout="30s",
            starttls=False,
            insecure_tls=False,
        )

        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_configuration(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "lease_id": "",
            "warnings": None,
            "wrap_info": None,
            "auth": None,
            "lease_duration": 0,
            "request_id": "dd7c3635-8e1c-d454-7381-bf11970fe8de",
            "data": {
                "anonymous_group_search": False,
                "binddn": "cn=admin,dc=example,dc=com",
                "case_sensitive_names": False,
                "certificate": "",
                "connection_timeout": "",
                "deny_null_bind": True,
                "dereference_aliases": "never",
                "discoverdn": False,
                "groupattr": "",
                "groupdn": "",
                "groupfilter": "",
                "insecure_tls": False,
                "max_page_size": "0",
                "request_timeout": "90",
                "starttls": False,
                "tls_max_version": "tls12",
                "tls_min_version": "tls12",
                "upndomain": "",
                "url": "ldaps://ldap.example.com",
                "userattr": "",
                "userdn": "",
            },
            "renewable": False,
        }
        mock_url = "http://localhost:8200/v1/{mount_point}/config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.read_config(
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_rotate_root(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/{mount_point}/rotate-root".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.rotate_root(
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT, "role1"),
            ("custom mount point", "other-ldap-tree", "role2"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_or_update_static_role(
        self, test_label, mount_point, name, requests_mocker
    ):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/{mount_point}/static-role/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.create_or_update_static_role(
            name=name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_static_role(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_name = "hvac"
        mock_response = {
            "lease_id": "",
            "warnings": None,
            "wrap_info": None,
            "auth": None,
            "lease_duration": 0,
            "request_id": "448bc87c-e948-ac5f-907c-9b01fb9d26c6",
            "data": {
                "username": "myuser",
                "dn": "cn=myuser,ou=users,dc=example,dc=com",
                "rotation_period": 600,
            },
            "renewable": False,
        }
        mock_url = "http://localhost:8200/v1/{mount_point}/static-role/{name}".format(
            mount_point=mount_point,
            name=role_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.read_static_role(
            name=role_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_generate_static_credentials(
        self, test_label, mount_point, requests_mocker
    ):
        expected_status_code = 200
        role_name = "hvac"
        mock_response = {
            "dn": "uid=hashicorp,ou=Users,dc=example,dc=com",
            "last_vault_rotation": "2020-02-19T11:31:53.7812-05:00",
            "password": "LTNfyn7pS7XEZIxEYQ2sEAWic02PEP7zSvIs0xMqIjaU0ORzLhKOKVmYLxL1Xkyv",
            "last_password": "?@09AZSen9TzUwK7ZhafS7B0GuWGraQjfWEna5SwnmF/tVaKFqjXhhGV/Z0v/pBJ",
            "rotation_period": 86400,
            "ttl": 86072,
            "username": "hashicorp",
        }
        mock_url = "http://localhost:8200/v1/{mount_point}/static-cred/{name}".format(
            mount_point=mount_point,
            name=role_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.generate_static_credentials(
            name=role_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_rotate_static_credentials(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        role_name = "hvac"
        mock_url = "http://localhost:8200/v1/{mount_point}/rotate-role/{name}".format(
            mount_point=mount_point,
            name=role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.rotate_static_credentials(
            name=role_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_delete_static_role(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        role_name = "hvac"
        mock_url = "http://localhost:8200/v1/{mount_point}/static-role/{name}".format(
            mount_point=mount_point,
            name=role_name,
        )
        requests_mocker.register_uri(
            method="DELETE",
            url=mock_url,
            status_code=expected_status_code,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.delete_static_role(
            name=role_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "other-ldap-tree"),
        ]
    )
    @requests_mock.Mocker()
    def test_list_static_roles(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "lease_id": "",
            "warnings": None,
            "wrap_info": None,
            "auth": None,
            "lease_duration": 0,
            "request_id": "0c34cc02-2f75-7deb-a531-33cf7434a729",
            "data": {
                "roles": [
                    {
                        "username": "myuser",
                        "dn": "cn=myuser,ou=users,dc=example,dc=com",
                        "rotation_period": 600,
                    }
                ]
            },
            "renewable": False,
        }
        mock_url = "http://localhost:8200/v1/{mount_point}/static-role".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        ldap = Ldap(adapter=JSONAdapter())
        response = ldap.list_static_roles(
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )



import pytest
import warnings

from unittest import mock

from hvac import exceptions
from hvac.utils import (
    generate_method_deprecation_message,
    generate_property_deprecation_message,
    generate_parameter_deprecation_message,
    aliased_parameter,
    comma_delimited_to_list,
    get_token_from_env,
    validate_list_of_strings_param,
    getattr_with_deprecated_properties,
    deprecated_method,
)


@pytest.fixture
def aliasable_func():
    def _func(pos0, pos1=None, *, kw0, kw1="kwone", kw2=None):
        return {
            "pos0": pos0,
            "pos1": pos1,
            "kw0": kw0,
            "kw1": kw1,
            "kw2": kw2,
        }

    return _func


@pytest.fixture
def deprecatable_method():
    def _meth():
        """docstring"""
        return mock.sentinel.dep_meth_retval

    return _meth


class ClassWithDeprecatedProperties:
    class _Sub:
        old1 = "new1"
        new1 = "NG"
        old2 = "old2"
        new2 = "new2"

    sub = _Sub()

    DEPRECATED_PROPERTIES = {
        "old1": dict(
            to_be_removed_in_version="99.88.77",
            client_property="sub",
        ),
        "old2": dict(
            to_be_removed_in_version="99.99.99",
            client_property="sub",
            new_property="new2",
        ),
    }


class TestUtils:
    @pytest.mark.parametrize("removed_in_version", ["99.88.77", "99.99.99"])
    def test_deprecated_method_no_new(self, removed_in_version, deprecatable_method):
        old_method = deprecatable_method

        with mock.patch(
            "hvac.utils.generate_method_deprecation_message",
            new=mock.Mock(return_value=mock.sentinel.dep_meth_msg),
        ) as gen_msg:
            wrapped = deprecated_method(
                to_be_removed_in_version=removed_in_version,
            )(old_method)

            gen_msg.assert_called_once_with(
                to_be_removed_in_version=removed_in_version,
                old_method_name=old_method.__name__,
                method_name=None,
                module_name=None,
            )
            assert wrapped.__doc__ == mock.sentinel.dep_meth_msg

        with mock.patch("warnings.warn") as warn:
            result = wrapped()
            warn.assert_called_once_with(
                message=mock.sentinel.dep_meth_msg,
                category=DeprecationWarning,
                stacklevel=2,
            )
            assert result == mock.sentinel.dep_meth_retval

    @pytest.mark.parametrize("removed_in_version", ["99.88.77", "99.99.99"])
    @pytest.mark.parametrize("new_meth_doc", [None, "newdoc"])
    def test_deprecated_method_new(
        self, removed_in_version, deprecatable_method, new_meth_doc
    ):
        old_method = deprecatable_method
        dep_meth_msg = "depmsg"

        def new_method():
            pass

        new_method.__doc__ = new_meth_doc

        with mock.patch(
            "hvac.utils.generate_method_deprecation_message",
            new=mock.Mock(return_value=dep_meth_msg),
        ) as gen_msg:
            wrapped = deprecated_method(
                to_be_removed_in_version=removed_in_version,
                new_method=new_method,
            )(old_method)

            gen_msg.assert_called_once_with(
                to_be_removed_in_version=removed_in_version,
                old_method_name=old_method.__name__,
                method_name=new_method.__name__,
                module_name=__name__,
            )
            if new_meth_doc is not None:
                assert new_meth_doc in wrapped.__doc__
            else:
                assert "N/A" in wrapped.__doc__

            assert dep_meth_msg in wrapped.__doc__
            assert (
                "Docstring content from this method's replacement copied below"
                in wrapped.__doc__
            )

        with mock.patch("warnings.warn") as warn:
            result = wrapped()
            warn.assert_called_once_with(
                message=dep_meth_msg,
                category=DeprecationWarning,
                stacklevel=2,
            )
            assert result == mock.sentinel.dep_meth_retval

    @pytest.mark.parametrize(
        ["item", "expected_new_prop", "expected_value", "expected_version"],
        [("old1", "old1", "new1", "99.88.77"), ("old2", "new2", "new2", "99.99.99")],
    )
    def test_getattr_with_deprecated_properties(
        self, item, expected_new_prop, expected_value, expected_version
    ):
        with mock.patch(
            "hvac.utils.generate_property_deprecation_message",
            new=mock.Mock(return_value=mock.sentinel.dep_prop_msg),
        ) as gen_msg, mock.patch("warnings.warn") as warn:
            result = getattr_with_deprecated_properties(
                ClassWithDeprecatedProperties,
                item,
                ClassWithDeprecatedProperties.DEPRECATED_PROPERTIES,
            )

            gen_msg.assert_called_once_with(
                to_be_removed_in_version=expected_version,
                old_name=item,
                new_name=expected_new_prop,
                new_attribute="sub",
            )
            warn.assert_called_once_with(
                message=mock.sentinel.dep_prop_msg,
                category=DeprecationWarning,
                stacklevel=2,
            )
            assert result == expected_value

    @pytest.mark.parametrize("item", ["old9", "old8"])
    def test_getattr_with_deprecated_properties_no_item(self, item):
        with pytest.raises(AttributeError, match=rf"has no attribute '{item}'"):
            getattr_with_deprecated_properties(
                ClassWithDeprecatedProperties,
                item,
                ClassWithDeprecatedProperties.DEPRECATED_PROPERTIES,
            )

    @pytest.mark.parametrize("token", ["token", "token2 ", " ", "\n"])
    def test_get_token_from_env_env_var(self, token):
        with mock.patch.dict("os.environ", {"VAULT_TOKEN": token}):
            with mock.patch("builtins.open", mock.mock_open()) as mopen:
                result = get_token_from_env()

                mopen.assert_not_called()
                assert result == token

    @mock.patch.dict("os.environ", clear=True)
    @mock.patch("os.path.expanduser", mock.Mock(return_value="/a/b/c/token"))
    @pytest.mark.parametrize("token", ["token", "token2 ", "", " ", "\n"])
    @pytest.mark.parametrize("exists", [True, False])
    def test_get_token_from_env_token_sink(self, token, exists):
        with mock.patch("os.path.exists", lambda x: exists):
            with mock.patch("builtins.open", mock.mock_open(read_data=token)) as mopen:
                result = get_token_from_env()

                if exists:
                    mopen.assert_called_once_with("/a/b/c/token")
                    if token.strip():
                        assert result == token.strip()
                    else:
                        assert result is None
                else:
                    mopen.assert_not_called()
                    assert result is None

    @pytest.mark.parametrize("param_name", ["PARAM1", "PARAM2"])
    @pytest.mark.parametrize(
        "param_argument", [None, [], ["1", "2"], "1,2,3", "", ",,"]
    )
    def test_validate_list_of_strings_param_pass(self, param_name, param_argument):
        result = validate_list_of_strings_param(param_name, param_argument)
        assert result is None

    @pytest.mark.parametrize("param_name", ["PARAM1", "PARAM2"])
    @pytest.mark.parametrize(
        "param_argument", [[None], [1], ["1", 2], ("1,2,3",), [["a"]]]
    )
    def test_validate_list_of_strings_param_fail(self, param_name, param_argument):
        with pytest.raises(exceptions.ParamValidationError) as e:
            validate_list_of_strings_param(param_name, param_argument)

        msg = str(e.value)
        assert str(param_name) in msg
        assert str(param_argument) in msg
        assert str(type(param_argument)) in msg

    @pytest.mark.parametrize(
        "list_param",
        [[], ["one"], [1, "two"], [1, "2", None], ["1", None, ["!", "@"], {}]],
    )
    def test_comma_delimited_to_list_from_list(self, list_param):
        result = comma_delimited_to_list(list_param=list_param)
        assert result == list_param

    @pytest.mark.parametrize(
        "list_param",
        [{}, {"a": 1}, None, 7, b"X,Y,Z"],
    )
    def test_comma_delimited_to_list_from_other(self, list_param):
        result = comma_delimited_to_list(list_param=list_param)
        assert result == []

    @pytest.mark.parametrize(
        ("list_param", "expected"),
        [
            ("", [""]),
            ("a", ["a"]),
            ("a,b,c", ["a", "b", "c"]),
            ("a, b, c", ["a", " b", " c"]),
            ("a,,c", ["a", "", "c"]),
        ],
    )
    def test_comma_delimited_to_list_from_str(self, list_param, expected):
        result = comma_delimited_to_list(list_param=list_param)
        assert result == expected

    @pytest.mark.parametrize("to_be_removed_in_version", ["99.0.0"])
    @pytest.mark.parametrize("old_name", ["old_one"])
    @pytest.mark.parametrize("new_name", ["new_one"])
    @pytest.mark.parametrize("new_attribute", ["new_attr"])
    @pytest.mark.parametrize("module_name", ["Client", "modulename"])
    def test_generate_property_deprecation_message(
        self,
        to_be_removed_in_version,
        old_name,
        new_name,
        new_attribute,
        module_name,
    ):
        result = generate_property_deprecation_message(
            to_be_removed_in_version=to_be_removed_in_version,
            old_name=old_name,
            new_name=new_name,
            new_attribute=new_attribute,
            module_name=module_name,
        )

        assert to_be_removed_in_version in result
        assert old_name in result
        assert new_name in result
        assert module_name in result

    @pytest.mark.parametrize("to_be_removed_in_version", ["99.0.0"])
    @pytest.mark.parametrize("old_method_name", ["old_one"])
    @pytest.mark.parametrize("method_name", [None, "new_one"])
    @pytest.mark.parametrize("module_name", [None, "modulename"])
    def test_generate_method_deprecation_message(
        self,
        to_be_removed_in_version,
        old_method_name,
        method_name,
        module_name,
    ):
        result = generate_method_deprecation_message(
            to_be_removed_in_version=to_be_removed_in_version,
            old_method_name=old_method_name,
            method_name=method_name,
            module_name=module_name,
        )

        assert to_be_removed_in_version in result
        assert old_method_name in result
        if method_name is not None and module_name is not None:
            assert method_name in result and module_name in result

    @pytest.mark.parametrize("to_be_removed_in_version", ["99.0.0"])
    @pytest.mark.parametrize("old_parameter_name", ["old_one"])
    @pytest.mark.parametrize("new_parameter_name", [None, "new_one"])
    @pytest.mark.parametrize("extra_notes", [None, "See also whatever."])
    def test_generate_parameter_deprecation_message(
        self,
        to_be_removed_in_version,
        old_parameter_name,
        new_parameter_name,
        extra_notes,
    ):
        result = generate_parameter_deprecation_message(
            to_be_removed_in_version=to_be_removed_in_version,
            old_parameter_name=old_parameter_name,
            new_parameter_name=new_parameter_name,
            extra_notes=extra_notes,
        )

        assert to_be_removed_in_version in result
        assert old_parameter_name in result
        assert (new_parameter_name is None) or (new_parameter_name in result)
        assert (extra_notes is None) or (extra_notes in result)

    @pytest.mark.parametrize("removed_in_version", ["abc", "123", None])
    @pytest.mark.parametrize(
        ["raise_on_multiple", "position", "i_args", "i_kwargs", "raises", "p1exp"],
        [
            (True, 1, ["p0"], {"kw0": "kw0", "alias8": "eight"}, None, "eight"),
            (True, None, ["p0"], {"kw0": "kw0", "alias8": "eight"}, None, "eight"),
            (
                True,
                1,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "p1"},
                ValueError,
                None,
            ),
            (
                True,
                None,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "p1"},
                ValueError,
                None,
            ),
            (
                True,
                1,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "p1"},
                ValueError,
                None,
            ),
            (
                True,
                None,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "p1"},
                ValueError,
                None,
            ),
            (
                True,
                1,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                ValueError,
                None,
            ),
            (
                True,
                None,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                ValueError,
                None,
            ),
            (
                True,
                1,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "peeone"},
                ValueError,
                None,
            ),
            (
                True,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "p1"},
                ValueError,
                None,
            ),
            (
                True,
                1,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight"},
                ValueError,
                None,
            ),
            (
                True,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight"},
                TypeError,
                None,
            ),
            (
                True,
                1,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "p1"},
                ValueError,
                None,
            ),
            (
                True,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "peeone"},
                ValueError,
                None,
            ),
            (
                True,
                1,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                ValueError,
                None,
            ),
            (
                True,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                ValueError,
                None,
            ),
            (True, 1, ["p0", "p1"], {"kw0": "kw0", "pos1": "peeone"}, TypeError, None),
            (
                True,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "pos1": "peeone"},
                TypeError,
                None,
            ),
            (True, 1, ["p0", "p1"], {"kw0": "kw0"}, None, "p1"),
            (True, None, ["p0", "p1"], {"kw0": "kw0"}, None, "p1"),
            (False, 1, ["p0"], {"kw0": "kw0", "alias8": "eight"}, None, "eight"),
            (False, None, ["p0"], {"kw0": "kw0", "alias8": "eight"}, None, "eight"),
            (
                False,
                1,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "p1"},
                None,
                "p1",
            ),
            (
                False,
                None,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "p1"},
                None,
                "p1",
            ),
            (
                False,
                1,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "p1"},
                None,
                "p1",
            ),
            (
                False,
                None,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "p1"},
                None,
                "p1",
            ),
            (
                False,
                1,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                None,
                "nine",
            ),
            (
                False,
                None,
                ["p0"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                None,
                "nine",
            ),
            (
                False,
                1,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "peeone"},
                TypeError,
                None,
            ),
            (
                False,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "pos1": "peeone"},
                TypeError,
                None,
            ),
            (False, 1, ["p0", "p1"], {"kw0": "kw0", "alias8": "eight"}, None, "p1"),
            (
                False,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight"},
                TypeError,
                None,
            ),
            (
                False,
                1,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "peeone"},
                TypeError,
                None,
            ),
            (
                False,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine", "pos1": "peeone"},
                TypeError,
                None,
            ),
            (
                False,
                1,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                None,
                "p1",
            ),
            (
                False,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "alias8": "eight", "alias9": "nine"},
                TypeError,
                None,
            ),
            (False, 1, ["p0", "p1"], {"kw0": "kw0", "pos1": "peeone"}, TypeError, None),
            (
                False,
                None,
                ["p0", "p1"],
                {"kw0": "kw0", "pos1": "peeone"},
                TypeError,
                None,
            ),
            (False, 1, ["p0", "p1"], {"kw0": "kw0"}, None, "p1"),
            (False, None, ["p0", "p1"], {"kw0": "kw0"}, None, "p1"),
        ],
    )
    def test_aliased_parameter(
        self,
        aliasable_func,
        removed_in_version,
        raise_on_multiple,
        i_args,
        i_kwargs,
        raises,
        p1exp,
        position,
    ):
        wrapped = aliased_parameter(
            "pos1",
            "alias9",
            "alias8",
            removed_in_version=removed_in_version,
            position=position,
            raise_on_multiple=raise_on_multiple,
        )(aliasable_func)

        alias_count = 0
        for a in i_kwargs.keys():
            if "alias" in a:
                alias_count += 1

        if raises is not None:
            with pytest.raises(raises, match=r"pos1"):
                if removed_in_version is None or alias_count == 0:
                    with warnings.catch_warnings():
                        warnings.simplefilter("error")
                        result = wrapped(*i_args, **i_kwargs)
                else:
                    with pytest.warns(DeprecationWarning) as wrecs:
                        result = wrapped(*i_args, **i_kwargs)
                        assert len(wrecs) == alias_count
                        for w in wrecs:
                            assert removed_in_version in str(w)
        else:
            if removed_in_version is None or alias_count == 0:
                with warnings.catch_warnings():
                    warnings.simplefilter("error")
                    result = wrapped(*i_args, **i_kwargs)
            else:
                with pytest.warns(DeprecationWarning) as wrecs:
                    result = wrapped(*i_args, **i_kwargs)
                    assert len(wrecs) == alias_count
                    for w in wrecs:
                        assert removed_in_version in str(w)

            assert result["pos0"] == "p0"
            assert result["pos1"] == p1exp
            assert result["kw0"] == "kw0"
            assert result["kw2"] is None
            assert "alias8" not in result
            assert "alias9" not in result



import logging
import requests_mock
from unittest import TestCase

from hvac.api.secrets_engines.database import Database
from hvac.api.secrets_engines.database import DEFAULT_MOUNT_POINT
from hvac.adapters import JSONAdapter


class TestDatabase(TestCase):
    def setUp(self):
        self.database = Database(adapter=JSONAdapter())
        self.expected_status_code = 200

    def mock_request(self, method, mock_url, mock_response):
        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=self.expected_status_code,
                json=mock_response,
            )
            return requests_mocker

    def test_configure(self):
        name = "test_db"
        plugin_name = "test_plugin"
        verify_connection = None
        allowed_roles = None
        root_rotation_statements = None
        mount_point = DEFAULT_MOUNT_POINT

        mock_url = f"http://localhost:8200/v1/{mount_point}/config/{name}"
        mock_response = {"status_code": 204}  # no response other than status code
        expected_status_code = 204

        with self.mock_request("POST", mock_url, mock_response):
            configure_response = self.database.configure(
                name=name,
                plugin_name=plugin_name,
                verify_connection=verify_connection,
                allowed_roles=allowed_roles,
                root_rotation_statements=root_rotation_statements,
                mount_point=mount_point,
            )
        logging.debug("configure_response: %s" % configure_response)

        self.assertEqual(configure_response["status_code"], expected_status_code)

    def test_rotate_root_credentials(self):
        name = "test_db"
        mount_point = DEFAULT_MOUNT_POINT

        mock_url = f"http://localhost:8200/v1/{mount_point}/rotate-root/{name}"
        mock_response = {"status_code": 204}  # no response other than status code
        expected_status_code = 204

        with self.mock_request("POST", mock_url, mock_response):
            rotate_root_credentials_response = self.database.rotate_root_credentials(
                name=name, mount_point=mount_point
            )
        logging.debug(
            "rotate_root_credentials_response: %s" % rotate_root_credentials_response
        )
        self.assertEqual(
            rotate_root_credentials_response["status_code"], expected_status_code
        )

    def test_rotate_static_role_credentials(self):
        name = "test_role"
        mount_point = DEFAULT_MOUNT_POINT

        mock_url = f"http://localhost:8200/v1/{mount_point}/rotate-role/{name}"
        mock_response = {
            "data": {"last_vault_rotation": "2023-09-25T19:02:38.347994635Z"}
        }

        with self.mock_request("POST", mock_url, mock_response):
            rotate_static_credentials_response = (
                self.database.rotate_static_role_credentials(
                    name=name, mount_point=mount_point
                )
            )
        logging.debug(
            "rotate_static_credentials_response: %s"
            % rotate_static_credentials_response
        )
        self.assertEqual(rotate_static_credentials_response, mock_response)



import pytest

from pytest_mock import MockFixture
from unittest.mock import MagicMock
from unittest import TestCase
import requests_mock
from parameterized import parameterized

from hvac import Client
from hvac.v1 import _sentinel, _smart_pop


class TestSmartPop:
    def test_smart_pop_duplicate(self):
        with pytest.raises(TypeError, match=r"got multiple values for argument"):
            _smart_pop(dict(a=5), "a", posvalue=9)

    def test_smart_pop_missing(self):
        with pytest.raises(
            TypeError, match=r"missing one required positional argument"
        ):
            _smart_pop(dict(a=5), "z")

    @pytest.mark.parametrize("dict", [{}, {"a": 2}])
    @pytest.mark.parametrize("default", [_sentinel, "other"])
    def test_smart_pop_pos_only(self, default, dict, mocker: MockFixture):
        result = _smart_pop(
            dict, "z", default=default, posvalue=mocker.sentinel.pos_only
        )
        assert result is mocker.sentinel.pos_only
        assert "z" not in dict

    @pytest.mark.parametrize("dict", [{}, {"a": 2}])
    def test_smart_pop_default_only(self, dict, mocker: MockFixture):
        result = _smart_pop(dict, "z", default=mocker.sentinel.default_only)
        assert result is mocker.sentinel.default_only
        assert "z" not in dict

    @pytest.mark.parametrize("dict", [{"a": 4, "b": 9}, {"a": 2}])
    def test_smart_pop_warns(self, dict):
        original = dict.copy()
        with pytest.warns(
            DeprecationWarning, match=r"https://github.com/hvac/hvac/issues/1034"
        ):
            result = _smart_pop(dict, "a")
        assert result == original["a"]
        assert "a" not in dict


class TestClientWriteData:
    test_url = "https://vault.example.com"
    test_path = "whatever/fake"
    response = dict(a=1, b="two")

    @pytest.fixture(autouse=True)
    def write_mock(self, requests_mock: requests_mock.Mocker):
        yield requests_mock.register_uri(
            method="POST",
            url=f"{self.test_url}/v1/{self.test_path}",
            json=self.response,
        )

    @pytest.fixture
    def client(self) -> Client:
        return Client(url=self.test_url)

    @pytest.mark.parametrize("wrap_ttl", [None, "3m"])
    def test_write_data(self, client: Client, wrap_ttl: str):
        response = client.write_data(self.test_path, data="cool", wrap_ttl=wrap_ttl)
        assert response == self.response


class TestOldClientWrite:
    test_url = "https://vault.example.com"
    test_path = "whatever/fake"

    @pytest.fixture(autouse=True)
    def mock_write_data(self, mocker: MockFixture) -> MagicMock:
        yield mocker.patch.object(Client, "write_data")

    @pytest.fixture
    def client(self) -> Client:
        return Client(url=self.test_url)

    @pytest.mark.parametrize("kwargs", [{}, {"wrap_ttl": "5m"}, {"other": 5}])
    def test_client_write_no_path(
        self,
        client: Client,
        mocker: MockFixture,
        kwargs: dict,
        mock_write_data: MagicMock,
    ):
        popper = mocker.patch("hvac.v1._smart_pop", new=mocker.Mock(wraps=_smart_pop))
        with pytest.raises(TypeError):
            client.write(**kwargs)
        popper.assert_called_once_with(mocker.ANY, "path", posvalue=_sentinel)
        mock_write_data.assert_not_called()

    @pytest.mark.parametrize("kwargs", [{}, {"other": 5}])
    def test_client_write_no_wrap_ttl(
        self,
        client: Client,
        mocker: MockFixture,
        kwargs: dict,
        mock_write_data: MagicMock,
    ):
        popper = mocker.patch("hvac.v1._smart_pop", new=mocker.Mock(wraps=_smart_pop))
        client.write(self.test_path, **kwargs)
        assert popper.call_count == 2
        expected_call = mocker.call(
            mocker.ANY, "wrap_ttl", default=None, posvalue=_sentinel
        )
        popper.assert_has_calls([expected_call])
        mock_write_data.assert_called_once_with(
            self.test_path, wrap_ttl=None, data=kwargs
        )

    def test_client_write_data_field(
        self, client: Client, mocker: MockFixture, mock_write_data: MagicMock
    ):
        with pytest.warns(
            PendingDeprecationWarning,
            match=r"argument 'data' was supplied as a keyword argument",
        ):
            client.write(self.test_path, data="thing")
        mock_write_data.assert_called_once_with(
            self.test_path, wrap_ttl=None, data=dict(data="thing")
        )


class TestSystemBackendMethods(TestCase):
    """Unit tests providing coverage for Vault system backend-related methods in the hvac Client class."""

    @parameterized.expand(
        [
            ("pki lease ID", "pki/issue/my-role/12c7e036-b59e-5e79-3370-03826fc6f34b"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_lease(self, test_label, test_lease_id, requests_mocker):
        test_path = "http://localhost:8200/v1/sys/leases/lookup"
        mock_response = {
            "issue_time": "2018-07-15T08:35:34.775859245-05:00",
            "renewable": False,
            "id": test_lease_id,
            "ttl": 259199,
            "expire_time": "2018-07-18T08:35:34.00004241-05:00",
            "last_renewal": None,
        }
        requests_mocker.register_uri(
            method="PUT",
            url=test_path,
            json=mock_response,
        )
        client = Client()
        response = client.sys.read_lease(
            lease_id=test_lease_id,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )



from unittest import TestCase

from parameterized import parameterized, param

from hvac import exceptions
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestAws(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "aws-test"

    def setUp(self):
        super().setUp()
        if "%s/" % self.TEST_MOUNT_POINT not in self.client.sys.list_auth_methods():
            self.client.sys.enable_auth_method(
                method_type="aws",
                path=self.TEST_MOUNT_POINT,
            )

    def tearDown(self):
        super().tearDown()
        self.client.sys.disable_auth_method(
            path=self.TEST_MOUNT_POINT,
        )

    @parameterized.expand(
        [
            param(
                "no params",
            ),
            param(
                "valid iam metadata input 1",
                iam_metadata="default",
            ),
            param(
                "valid iam metadata input 2",
                iam_metadata=["auth_type", "client_arn", "inferred_aws_region"],
            ),
            param(
                "valid ec2 metadata input 1",
                ec2_metadata=["region", "ami_id", "account_id"],
            ),
            param("valid ec2 metadata input 2", ec2_metadata="default"),
            param("valid ec2 alias input 1", ec2_alias="instance_id"),
            param("valid ec2 alias input 2", ec2_alias="role_id"),
            param("valid iam alias input 1", iam_alias="full_arn"),
            param("valid iam alias input 2", iam_alias="role_id"),
            param(
                "valid combination",
                ec2_metadata=["region", "instance_id", "auth_type"],
                iam_metadata=[
                    "inferred_entity_type",
                    "inferred_entity_id",
                    "canonical_arn",
                    "client_user_id",
                    "account_id",
                ],
                ec2_alias="image_id",
                iam_alias="unique_id",
            ),
        ]
    )
    def test_configure_identity_integration_succeeds(
        self, label, ec2_metadata="", iam_metadata="", ec2_alias=None, iam_alias=None
    ):
        configure_response = self.client.auth.aws.configure_identity_integration(
            mount_point=self.TEST_MOUNT_POINT,
            ec2_metadata=ec2_metadata,
            iam_metadata=iam_metadata,
            ec2_alias=ec2_alias,
            iam_alias=iam_alias,
        )
        self.assertEqual(
            first=bool(configure_response),
            second=True,
        )

    @parameterized.expand(
        [
            param(
                "invalid ec2 metadata",
                raises=exceptions.InvalidRequest,
                exception_message="contains an unavailable field, please select from",
                ec2_metadata="something invalid",
            ),
            param(
                "invalid iam metadata",
                iam_metadata="something invalid",
                raises=exceptions.InvalidRequest,
                exception_message="contains an unavailable field, please select from",
            ),
            param(
                "invalid iam alias",
                iam_alias="something invalid",
                raises=exceptions.ParamValidationError,
                exception_message="invalid iam alias type provided",
            ),
            param(
                "invalid ec2 alias",
                ec2_alias="something invalid",
                raises=exceptions.ParamValidationError,
                exception_message="invalid ec2 alias type provided",
            ),
        ]
    )
    def test_configure_identity_integration_fails(
        self,
        label,
        raises,
        exception_message,
        ec2_metadata=None,
        iam_metadata=None,
        ec2_alias=None,
        iam_alias=None,
    ):
        with self.assertRaises(raises) as cm:
            self.client.auth.aws.configure_identity_integration(
                mount_point=self.TEST_MOUNT_POINT,
                ec2_metadata=ec2_metadata,
                iam_metadata=iam_metadata,
                ec2_alias=ec2_alias,
                iam_alias=iam_alias,
            )
        self.assertIn(
            member=exception_message,
            container=str(cm.exception),
        )



import json
import logging
from unittest import TestCase, skipIf

from parameterized import parameterized, param
from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestPolicies(HvacIntegrationTestCase, TestCase):
    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(backend_type="kv", path="test")

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path="test")
        super().tearDown()

    def test_create_acl_policy(self):
        policy_dict = {
            "name": "test-acl-policy",
            "policy": 'path "sys/health" { capabilities = ["read", "sudo"]}',
        }

        # Create policy
        create_or_update_policy_response = self.client.sys.create_or_update_acl_policy(
            name=policy_dict["name"], policy=policy_dict["policy"]
        )
        logging.debug(
            "create_or_update_policy_response: %s" % create_or_update_policy_response
        )

        self.assertEqual(
            first=policy_dict,
            second=self.client.sys.read_acl_policy(policy_dict["name"])["data"],
        )

    @parameterized.expand(
        [
            param(
                "pretty",
                pretty_print=True,
            ),
            param(
                "compact",
                pretty_print=False,
            ),
        ]
    )
    def test_create_acl_policy_dict(self, label, pretty_print):
        dict_policy = {
            "path": {
                "sys/health": {
                    "capabilities": ["read", "sudo"],
                },
            },
        }
        policy_dict = {
            "name": "test-acl-policy-dict",
            "policy": dict_policy,
        }

        # Create policy
        create_or_update_policy_response = self.client.sys.create_or_update_acl_policy(
            name=policy_dict["name"], policy=dict_policy, pretty_print=pretty_print
        )
        logging.debug(
            "create_or_update_policy_response: %s" % create_or_update_policy_response
        )

        policy_read_response = self.client.sys.read_acl_policy(policy_dict["name"])
        json_line_count = len(policy_read_response["data"]["policy"].splitlines())

        self.assertEqual(
            first=policy_dict["name"],
            second=policy_read_response["data"]["name"],
        )

        self.assertDictEqual(
            d1=json.loads(policy_read_response["data"]["policy"]),
            d2=policy_dict["policy"],
        )

        if pretty_print:
            self.assertGreater(
                a=json_line_count,
                b=1,
            )
        else:
            self.assertEqual(
                first=json_line_count,
                second=1,
            )

    def test_update_acl_policy(self):
        policy_dict = {
            "name": "test-acl-policy",
            "policy": 'path "sys/health" { capabilities = ["read", "sudo"]}',
        }

        # Create policy
        self.client.sys.create_or_update_acl_policy(
            name=policy_dict["name"],
            policy='path "sys/health" { capabilities = ["read"]}',
        )

        # Update policy
        self.client.sys.create_or_update_acl_policy(
            name=policy_dict["name"], policy=policy_dict["policy"]
        )

        self.assertEqual(
            first=policy_dict,
            second=self.client.sys.read_acl_policy(policy_dict["name"])["data"],
        )

    def test_read_acl_policy(self):
        policy_dict = {
            "name": "test-acl-policy",
            "policy": 'path "sys/health" { capabilities = ["read", "sudo"]}',
        }

        self.client.sys.create_or_update_acl_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
        )

        # Read the policy that was just created
        read_acl_policy_response = self.client.sys.read_acl_policy(
            name=policy_dict["name"],
        )
        logging.debug("read_acl_policy_response: %s" % read_acl_policy_response)

        self.assertEqual(
            first=policy_dict,
            second=read_acl_policy_response["data"],
        )

    def test_list_acl_policy(self):
        policy_dict = {
            "name": "test-acl-policy",
            "policy": 'path "sys/health" { capabilities = ["read", "sudo"]}',
        }

        self.client.sys.create_or_update_acl_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
        )

        list_acl_policies_response = self.client.sys.list_acl_policies()
        logging.debug("list_acl_policies_response: %s" % list_acl_policies_response)

        self.assertIn(
            member=policy_dict["name"],
            container=list_acl_policies_response["data"]["keys"],
        )

    def test_delete_acl_policy(self):
        policy_dict = {
            "name": "test-acl-policy",
            "policy": 'path "sys/health" { capabilities = ["read", "sudo"]}',
        }

        self.client.sys.create_or_update_acl_policy(
            name=policy_dict["name"], policy=policy_dict["policy"]
        )

        # Delete the policy that was just created
        delete_acl_policy_response = self.client.sys.delete_acl_policy(
            name=policy_dict["name"],
        )

        logging.debug("delete_acl_policy_response: %s" % delete_acl_policy_response)

        with self.assertRaises(exceptions.InvalidPath):
            self.client.sys.read_acl_policy(
                name=policy_dict["name"],
            )

    @skipIf(
        not utils.is_enterprise(), "RGP policies only supported with Enterprise Vault"
    )
    def test_create_rgp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-rgp-policy",
            "policy": policy,
        }

        # Create policy
        create_or_update_policy_response = self.client.sys.create_or_update_rgp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
        )
        logging.debug(
            "create_or_update_policy_response: %s" % create_or_update_policy_response
        )

        self.assertEqual(
            first=policy_dict,
            second=self.client.sys.read_rgp_policy(policy_dict["name"])["data"],
        )

    @skipIf(
        not utils.is_enterprise(), "RGP policies only supported with Enterprise Vault"
    )
    def test_update_rgp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-rgp-policy",
            "policy": policy,
        }

        # Create policy
        self.client.sys.create_or_update_rgp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level="hard-mandatory",
        )

        # Update policy
        self.client.sys.create_or_update_rgp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
        )

        self.assertEqual(
            first=policy_dict,
            second=self.client.sys.read_rgp_policy(policy_dict["name"])["data"],
        )

    @skipIf(
        not utils.is_enterprise(), "RGP policies only supported with Enterprise Vault"
    )
    def test_read_rgp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-rgp-policy",
            "policy": policy,
        }

        self.client.sys.create_or_update_rgp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
        )

        # Read the policy that was just created
        read_rgp_policy_response = self.client.sys.read_rgp_policy(
            name=policy_dict["name"],
        )
        logging.debug("read_rgp_policy_response: %s" % read_rgp_policy_response)

        self.assertEqual(
            first=policy_dict,
            second=read_rgp_policy_response["data"],
        )

    @skipIf(
        not utils.is_enterprise(), "RGP policies only supported with Enterprise Vault"
    )
    def test_list_rgp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-rgp-policy",
            "policy": policy,
        }

        self.client.sys.create_or_update_rgp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
        )

        list_rgp_policies_response = self.client.sys.list_rgp_policies()
        logging.debug("list_rgp_policies_response: %s" % list_rgp_policies_response)

        self.assertIn(
            member=policy_dict["name"],
            container=list_rgp_policies_response["data"]["keys"],
        )

    @skipIf(
        not utils.is_enterprise(), "RGP policies only supported with Enterprise Vault"
    )
    def test_delete_rgp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-rgp-policy",
            "policy": policy,
        }

        self.client.sys.create_or_update_rgp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
        )

        # Delete the policy that was just created
        delete_rgp_policy_response = self.client.sys.delete_rgp_policy(
            name=policy_dict["name"],
        )

        logging.debug("delete_rgp_policy_response: %s" % delete_rgp_policy_response)

        with self.assertRaises(exceptions.InvalidPath):
            self.client.sys.read_rgp_policy(
                name=policy_dict["name"],
            )

    @skipIf(
        not utils.is_enterprise(), "EGP policies only supported with Enterprise Vault"
    )
    def test_create_egp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-egp-policy",
            "policy": policy,
            "paths": ["/test"],
        }

        # Create policy
        create_or_update_policy_response = self.client.sys.create_or_update_egp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
            paths=policy_dict["paths"],
        )
        logging.debug(
            "create_or_update_policy_response: %s" % create_or_update_policy_response
        )

        self.assertEqual(
            first=policy_dict,
            second=self.client.sys.read_egp_policy(policy_dict["name"])["data"],
        )

    @skipIf(
        not utils.is_enterprise(), "EGP policies only supported with Enterprise Vault"
    )
    def test_update_egp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-egp-policy",
            "policy": policy,
            "paths": ["/test"],
        }

        # Create policy
        self.client.sys.create_or_update_egp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level="hard-mandatory",
            paths=policy_dict["paths"],
        )

        # Update policy
        self.client.sys.create_or_update_egp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
            paths=policy_dict["paths"],
        )

        self.assertEqual(
            first=policy_dict,
            second=self.client.sys.read_egp_policy(policy_dict["name"])["data"],
        )

    @skipIf(
        not utils.is_enterprise(), "EGP policies only supported with Enterprise Vault"
    )
    def test_read_egp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-egp-policy",
            "policy": policy,
            "paths": ["/test"],
        }

        self.client.sys.create_or_update_egp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
            paths=policy_dict["paths"],
        )

        # Read the policy that was just created
        read_egp_policy_response = self.client.sys.read_egp_policy(
            name=policy_dict["name"],
        )
        logging.debug("read_egp_policy_response: %s" % read_egp_policy_response)

        self.assertEqual(
            first=policy_dict,
            second=read_egp_policy_response["data"],
        )

    @skipIf(
        not utils.is_enterprise(), "EGP policies only supported with Enterprise Vault"
    )
    def test_list_egp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-egp-policy",
            "policy": policy,
            "paths": ["/test"],
        }

        self.client.sys.create_or_update_egp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
            paths=policy_dict["paths"],
        )

        list_egp_policies_response = self.client.sys.list_egp_policies()
        logging.debug("list_egp_policies_response: %s" % list_egp_policies_response)

        self.assertIn(
            member=policy_dict["name"],
            container=list_egp_policies_response["data"]["keys"],
        )

    @skipIf(
        not utils.is_enterprise(), "EGP policies only supported with Enterprise Vault"
    )
    def test_delete_egp_policy(self):
        policy = """import "time"
        import "strings"

        main = rule when not strings.has_prefix(request.path, "auth/ldap/login") {
            time.load(token.creation_time).unix > time.load("2017-09-17T13:25:29Z").unix
        }
        """
        policy_dict = {
            "enforcement_level": "soft-mandatory",
            "name": "test-egp-policy",
            "policy": policy,
            "paths": ["/test"],
        }

        self.client.sys.create_or_update_egp_policy(
            name=policy_dict["name"],
            policy=policy_dict["policy"],
            enforcement_level=policy_dict["enforcement_level"],
            paths=policy_dict["paths"],
        )

        # Delete the policy that was just created
        delete_egp_policy_response = self.client.sys.delete_egp_policy(
            name=policy_dict["name"],
        )

        logging.debug("delete_egp_policy_response: %s" % delete_egp_policy_response)

        with self.assertRaises(exceptions.InvalidPath):
            self.client.sys.read_egp_policy(
                name=policy_dict["name"],
            )



import pytest

from unittest import mock

from hvac.adapters import Adapter


class MockAdapter(Adapter):
    def __init__(self, *args, **kwargs):
        if "session" not in kwargs:
            kwargs["session"] = mock.MagicMock()
        super().__init__(*args, **kwargs)

    def request(self, *args, **kwargs):
        return (args, kwargs)

    def get_login_token(self, response):
        raise NotImplementedError()


@pytest.fixture
def mock_adapter():
    adapter = MockAdapter()
    with mock.patch.object(adapter, "request", mock.Mock(wraps=MockAdapter.request)):
        yield adapter


@pytest.fixture
def mock_warn():
    with mock.patch("warnings.warn") as warn:
        yield warn
